import{s as Ie,n as We,o as De}from"../chunks/scheduler.7c46a39e.js";import{S as Oe,i as Ve,g as o,s as l,r as E,A as Ge,h as r,f as a,c as n,j as Fe,u as S,x as s,k as Le,y as Je,a as i,v as B,d as F,t as I,w as W}from"../chunks/index.fa0f2890.js";import{H as D}from"../chunks/Heading.36c897de.js";function Ke(_e){let d,O,q,V,p,G,f,ye=`This conceptual guide gives a brief overview of <a href="https://arxiv.org/abs/2106.09685" rel="nofollow">LoRA</a>, a technique that accelerates
the fine-tuning of large models while consuming less memory.`,J,h,be=`To make fine-tuning more efficient, LoRA’s approach is to represent the weight updates with two smaller
matrices (called <strong>update matrices</strong>) through low-rank decomposition. These new matrices can be trained to adapt to the
new data while keeping the overall number of changes low. The original weight matrix remains frozen and doesn’t receive
any further adjustments. To produce the final results, both the original and the adapted weights are combined.`,K,c,ve="This approach has a number of advantages:",N,u,Te="<li>LoRA makes fine-tuning more efficient by drastically reducing the number of trainable parameters.</li> <li>The original pre-trained weights are kept frozen, which means you can have multiple lightweight and portable LoRA models for various downstream tasks built on top of them.</li> <li>LoRA is orthogonal to many other parameter-efficient methods and can be combined with many of them.</li> <li>Performance of models fine-tuned using LoRA is comparable to the performance of fully fine-tuned models.</li> <li>LoRA does not add any inference latency because adapter weights can be merged with the base model.</li>",Q,g,xe=`In principle, LoRA can be applied to any subset of weight matrices in a neural network to reduce the number of trainable
parameters. However, for simplicity and further parameter efficiency, in Transformer models LoRA is typically applied to
attention blocks only. The resulting number of trainable parameters in a LoRA model depends on the size of the low-rank
update matrices, which is determined mainly by the rank <code>r</code> and the shape of the original weight matrix.`,X,w,Y,L,Me='While LoRA is significantly smaller and faster to train, you may encounter latency issues during inference due to separately loading the base model and the LoRA model. To eliminate latency, use the <a href="/docs/peft/main/en/package_reference/lora#peft.LoraModel.merge_and_unload">merge_and_unload()</a> function to merge the adapter weights with the base model which allows you to effectively use the newly merged model as a standalone model.',Z,m,$e='<img src="https://huggingface.co/datasets/huggingface/documentation-images/resolve/main/peft/lora_diagram.png"/>',ee,_,Ae="This works because during training, the smaller weight matrices (<em>A</em> and <em>B</em> in the diagram above) are separate. But once training is complete, the weights can actually be merged into a new weight matrix that is identical.",te,y,ae,b,ke=`Use <a href="/docs/peft/main/en/package_reference/tuners#peft.tuners.tuners_utils.BaseTuner.merge_adapter">merge_adapter()</a> to merge the LoRa layers into the base model while retaining the PeftModel.
This will help in later unmerging, deleting, loading different adapters and so on.`,ie,v,He=`Use <a href="/docs/peft/main/en/package_reference/tuners#peft.tuners.tuners_utils.BaseTuner.unmerge_adapter">unmerge_adapter()</a> to unmerge the LoRa layers from the base model while retaining the PeftModel.
This will help in later merging, deleting, loading different adapters and so on.`,le,T,Re=`Use <a href="/docs/peft/main/en/package_reference/lora#peft.LoraModel.unload">unload()</a> to get back the base model without the merging of the active lora modules.
This will help when you want to get back the pretrained base model in some applications when you want to reset the model to its original state.
For example, in Stable Diffusion WebUi, when the user wants to infer with base model post trying out LoRAs.`,ne,x,Ce='Use <a href="/docs/peft/main/en/package_reference/lora#peft.LoraModel.delete_adapter">delete_adapter()</a> to delete an existing adapter.',oe,M,Pe='Use <a href="/docs/peft/main/en/package_reference/lora#peft.LoraModel.add_weighted_adapter">add_weighted_adapter()</a> to combine multiple LoRAs into a new adapter based on the user provided weighing scheme.',re,$,se,A,Ue="As with other methods supported by PEFT, to fine-tune a model using LoRA, you need to:",de,k,je="<li>Instantiate a base model.</li> <li>Create a configuration (<code>LoraConfig</code>) where you define LoRA-specific parameters.</li> <li>Wrap the base model with <code>get_peft_model()</code> to get a trainable <code>PeftModel</code>.</li> <li>Train the <code>PeftModel</code> as you normally would train the base model.</li>",me,H,qe="<code>LoraConfig</code> allows you to control how LoRA is applied to the base model through the following parameters:",pe,R,ze="<li><code>r</code>: the rank of the update matrices, expressed in <code>int</code>. Lower rank results in smaller update matrices with fewer trainable parameters.</li> <li><code>target_modules</code>: The modules (for example, attention blocks) to apply the LoRA update matrices.</li> <li><code>alpha</code>: LoRA scaling factor.</li> <li><code>bias</code>: Specifies if the <code>bias</code> parameters should be trained. Can be <code>&#39;none&#39;</code>, <code>&#39;all&#39;</code> or <code>&#39;lora_only&#39;</code>.</li> <li><code>modules_to_save</code>: List of modules apart from LoRA layers to be set as trainable and saved in the final checkpoint. These typically include model’s custom head that is randomly initialized for the fine-tuning task.</li> <li><code>layers_to_transform</code>: List of layers to be transformed by LoRA. If not specified, all layers in <code>target_modules</code> are transformed.</li> <li><code>layers_pattern</code>: Pattern to match layer names in <code>target_modules</code>, if <code>layers_to_transform</code> is specified. By default <code>PeftModel</code> will look at common layer pattern (<code>layers</code>, <code>h</code>, <code>blocks</code>, etc.), use it for exotic and custom models.</li> <li><code>rank_pattern</code>: The mapping from layer names or regexp expression to ranks which are different from the default rank specified by <code>r</code>.</li> <li><code>alpha_pattern</code>: The mapping from layer names or regexp expression to alphas which are different from the default alpha specified by <code>lora_alpha</code>.</li>",fe,C,he,P,Ee="For an example of LoRA method application to various downstream tasks, please refer to the following guides:",ce,U,Se='<li><a href="../task_guides/image_classification_lora">Image classification using LoRA</a></li> <li><a href="../task_guides/semantic_segmentation_lora">Semantic segmentation</a></li>',ue,j,Be=`While the original paper focuses on language models, the technique can be applied to any dense layers in deep learning
models. As such, you can leverage this technique with diffusion models. See <a href="../task_guides/task_guides/dreambooth_lora">Dreambooth fine-tuning with LoRA</a> task guide for an example.`,ge,z,we;return p=new D({props:{title:"LoRA",local:"lora",headingTag:"h1"}}),w=new D({props:{title:"Merge LoRA weights into the base model",local:"merge-lora-weights-into-the-base-model",headingTag:"h2"}}),y=new D({props:{title:"Utils for LoRA",local:"utils-for-lora",headingTag:"h2"}}),$=new D({props:{title:"Common LoRA parameters in PEFT",local:"common-lora-parameters-in-peft",headingTag:"h2"}}),C=new D({props:{title:"LoRA examples",local:"lora-examples",headingTag:"h2"}}),{c(){d=o("meta"),O=l(),q=o("p"),V=l(),E(p.$$.fragment),G=l(),f=o("p"),f.innerHTML=ye,J=l(),h=o("p"),h.innerHTML=be,K=l(),c=o("p"),c.textContent=ve,N=l(),u=o("ul"),u.innerHTML=Te,Q=l(),g=o("p"),g.innerHTML=xe,X=l(),E(w.$$.fragment),Y=l(),L=o("p"),L.innerHTML=Me,Z=l(),m=o("div"),m.innerHTML=$e,ee=l(),_=o("p"),_.innerHTML=Ae,te=l(),E(y.$$.fragment),ae=l(),b=o("p"),b.innerHTML=ke,ie=l(),v=o("p"),v.innerHTML=He,le=l(),T=o("p"),T.innerHTML=Re,ne=l(),x=o("p"),x.innerHTML=Ce,oe=l(),M=o("p"),M.innerHTML=Pe,re=l(),E($.$$.fragment),se=l(),A=o("p"),A.textContent=Ue,de=l(),k=o("ol"),k.innerHTML=je,me=l(),H=o("p"),H.innerHTML=qe,pe=l(),R=o("ul"),R.innerHTML=ze,fe=l(),E(C.$$.fragment),he=l(),P=o("p"),P.textContent=Ee,ce=l(),U=o("ul"),U.innerHTML=Se,ue=l(),j=o("p"),j.innerHTML=Be,ge=l(),z=o("p"),this.h()},l(e){const t=Ge("svelte-u9bgzb",document.head);d=r(t,"META",{name:!0,content:!0}),t.forEach(a),O=n(e),q=r(e,"P",{}),Fe(q).forEach(a),V=n(e),S(p.$$.fragment,e),G=n(e),f=r(e,"P",{"data-svelte-h":!0}),s(f)!=="svelte-10j2kq5"&&(f.innerHTML=ye),J=n(e),h=r(e,"P",{"data-svelte-h":!0}),s(h)!=="svelte-yvfnx3"&&(h.innerHTML=be),K=n(e),c=r(e,"P",{"data-svelte-h":!0}),s(c)!=="svelte-1usho35"&&(c.textContent=ve),N=n(e),u=r(e,"UL",{"data-svelte-h":!0}),s(u)!=="svelte-b91w2n"&&(u.innerHTML=Te),Q=n(e),g=r(e,"P",{"data-svelte-h":!0}),s(g)!=="svelte-1pnerbz"&&(g.innerHTML=xe),X=n(e),S(w.$$.fragment,e),Y=n(e),L=r(e,"P",{"data-svelte-h":!0}),s(L)!=="svelte-1lsy0sj"&&(L.innerHTML=Me),Z=n(e),m=r(e,"DIV",{class:!0,"data-svelte-h":!0}),s(m)!=="svelte-1f6iuw5"&&(m.innerHTML=$e),ee=n(e),_=r(e,"P",{"data-svelte-h":!0}),s(_)!=="svelte-6quaur"&&(_.innerHTML=Ae),te=n(e),S(y.$$.fragment,e),ae=n(e),b=r(e,"P",{"data-svelte-h":!0}),s(b)!=="svelte-u1h9r6"&&(b.innerHTML=ke),ie=n(e),v=r(e,"P",{"data-svelte-h":!0}),s(v)!=="svelte-121m51a"&&(v.innerHTML=He),le=n(e),T=r(e,"P",{"data-svelte-h":!0}),s(T)!=="svelte-12mnxsd"&&(T.innerHTML=Re),ne=n(e),x=r(e,"P",{"data-svelte-h":!0}),s(x)!=="svelte-33nb0p"&&(x.innerHTML=Ce),oe=n(e),M=r(e,"P",{"data-svelte-h":!0}),s(M)!=="svelte-v3as9t"&&(M.innerHTML=Pe),re=n(e),S($.$$.fragment,e),se=n(e),A=r(e,"P",{"data-svelte-h":!0}),s(A)!=="svelte-kdwy7m"&&(A.textContent=Ue),de=n(e),k=r(e,"OL",{"data-svelte-h":!0}),s(k)!=="svelte-yij1ed"&&(k.innerHTML=je),me=n(e),H=r(e,"P",{"data-svelte-h":!0}),s(H)!=="svelte-ezuo7y"&&(H.innerHTML=qe),pe=n(e),R=r(e,"UL",{"data-svelte-h":!0}),s(R)!=="svelte-vpooj9"&&(R.innerHTML=ze),fe=n(e),S(C.$$.fragment,e),he=n(e),P=r(e,"P",{"data-svelte-h":!0}),s(P)!=="svelte-aryync"&&(P.textContent=Ee),ce=n(e),U=r(e,"UL",{"data-svelte-h":!0}),s(U)!=="svelte-19vcsei"&&(U.innerHTML=Se),ue=n(e),j=r(e,"P",{"data-svelte-h":!0}),s(j)!=="svelte-1moq18d"&&(j.innerHTML=Be),ge=n(e),z=r(e,"P",{}),Fe(z).forEach(a),this.h()},h(){Le(d,"name","hf:doc:metadata"),Le(d,"content",Ne),Le(m,"class","flex justify-center")},m(e,t){Je(document.head,d),i(e,O,t),i(e,q,t),i(e,V,t),B(p,e,t),i(e,G,t),i(e,f,t),i(e,J,t),i(e,h,t),i(e,K,t),i(e,c,t),i(e,N,t),i(e,u,t),i(e,Q,t),i(e,g,t),i(e,X,t),B(w,e,t),i(e,Y,t),i(e,L,t),i(e,Z,t),i(e,m,t),i(e,ee,t),i(e,_,t),i(e,te,t),B(y,e,t),i(e,ae,t),i(e,b,t),i(e,ie,t),i(e,v,t),i(e,le,t),i(e,T,t),i(e,ne,t),i(e,x,t),i(e,oe,t),i(e,M,t),i(e,re,t),B($,e,t),i(e,se,t),i(e,A,t),i(e,de,t),i(e,k,t),i(e,me,t),i(e,H,t),i(e,pe,t),i(e,R,t),i(e,fe,t),B(C,e,t),i(e,he,t),i(e,P,t),i(e,ce,t),i(e,U,t),i(e,ue,t),i(e,j,t),i(e,ge,t),i(e,z,t),we=!0},p:We,i(e){we||(F(p.$$.fragment,e),F(w.$$.fragment,e),F(y.$$.fragment,e),F($.$$.fragment,e),F(C.$$.fragment,e),we=!0)},o(e){I(p.$$.fragment,e),I(w.$$.fragment,e),I(y.$$.fragment,e),I($.$$.fragment,e),I(C.$$.fragment,e),we=!1},d(e){e&&(a(O),a(q),a(V),a(G),a(f),a(J),a(h),a(K),a(c),a(N),a(u),a(Q),a(g),a(X),a(Y),a(L),a(Z),a(m),a(ee),a(_),a(te),a(ae),a(b),a(ie),a(v),a(le),a(T),a(ne),a(x),a(oe),a(M),a(re),a(se),a(A),a(de),a(k),a(me),a(H),a(pe),a(R),a(fe),a(he),a(P),a(ce),a(U),a(ue),a(j),a(ge),a(z)),a(d),W(p,e),W(w,e),W(y,e),W($,e),W(C,e)}}}const Ne='{"title":"LoRA","local":"lora","sections":[{"title":"Merge LoRA weights into the base model","local":"merge-lora-weights-into-the-base-model","sections":[],"depth":2},{"title":"Utils for LoRA","local":"utils-for-lora","sections":[],"depth":2},{"title":"Common LoRA parameters in PEFT","local":"common-lora-parameters-in-peft","sections":[],"depth":2},{"title":"LoRA examples","local":"lora-examples","sections":[],"depth":2}],"depth":1}';function Qe(_e){return De(()=>{new URLSearchParams(window.location.search).get("fw")}),[]}class et extends Oe{constructor(d){super(),Ve(this,d,Qe,Ke,Ie,{})}}export{et as component};
